set linesize 32000
set pagesize 1000
set echo on
set termout on
set trimspool on

spool januarycheese.log

set define off

drop table sales;
drop table products;

create table sales
(
sale_date date,
product_number number,
customer_number number,
amount number
);

create table products
(
product_number number,
product_type varchar2(12),
product_name varchar2(12)
);

insert into sales values (to_date('01/02/2012','MM/DD/YYYY'),1,1,100);
insert into sales values (to_date('01/03/2012','MM/DD/YYYY'),1,1,200);
insert into sales values (to_date('02/04/2012','MM/DD/YYYY'),2,2,300);
insert into sales values (to_date('02/05/2012','MM/DD/YYYY'),2,2,400);

insert into products values (1,'Meat','Ground Beef');
insert into products values (2,'Cheese','Chedder');

commit;

-- gather stats with max bucket histograms all columns and all rows

execute DBMS_STATS.SET_TABLE_PREFS(NULL,'SALES','ESTIMATE_PERCENT','100');
execute DBMS_STATS.SET_TABLE_PREFS(NULL,'PRODUCTS','ESTIMATE_PERCENT','100');
execute DBMS_STATS.SET_TABLE_PREFS(NULL,'SALES','METHOD_OPT','FOR ALL COLUMNS SIZE 254');
execute DBMS_STATS.SET_TABLE_PREFS(NULL,'PRODUCTS','METHOD_OPT','FOR ALL COLUMNS SIZE 254');
select DBMS_STATS.CREATE_EXTENDED_STATS (NULL,'SALES','(SALE_DATE,PRODUCT_NUMBER)') from dual;
execute dbms_stats.gather_table_stats(NULL,'SALES');
execute dbms_stats.gather_table_stats(NULL,'PRODUCTS');

-- show column statistics

column table_name format a30
column column_name format a30
column lo format a30
column hi format a30
column data_default format a30

select tc.table_name,column_name,
decode(data_type,'NUMBER',to_char(utl_raw.cast_to_number(low_value))
,'VARCHAR2',utl_raw.cast_to_varchar2(low_value)
,'CHAR',utl_raw.cast_to_varchar2(low_value)
,'DATE',
(to_number(substr( low_value, 1, 2 ), 'xx')-100)*100+
(to_number(substr( low_value, 3, 2 ), 'xx' )-100)|| '/' ||
to_number(substr( low_value, 5, 2 ), 'xx' ) || '/' ||
to_number(substr( low_value, 7, 2 ), 'xx' ) || ' ' ||
(to_number(substr( low_value, 9, 2 ), 'xx' )-1) || ':' ||
(to_number(substr( low_value,11, 2 ), 'xx' )-1) || ':' ||
(to_number(substr( low_value,13, 2 ), 'xx' )-1)
,'Hi') lo,
decode(data_type,'NUMBER',to_char(utl_raw.cast_to_number(high_value))
,'VARCHAR2',utl_raw.cast_to_varchar2(high_value)
,'CHAR',utl_raw.cast_to_varchar2(high_value)
,'DATE',
(to_number(substr( high_value, 1, 2 ), 'xx')-100)*100+
(to_number(substr( high_value, 3, 2 ), 'xx' )-100)|| '/' ||
to_number(substr( high_value, 5, 2 ), 'xx' ) || '/' ||
to_number(substr( high_value, 7, 2 ), 'xx' ) || ' ' ||
(to_number(substr( high_value, 9, 2 ), 'xx' )-1) || ':' ||
(to_number(substr( high_value,11, 2 ), 'xx' )-1) || ':' ||
(to_number(substr( high_value,13, 2 ), 'xx' )-1)
,'Hi') hi,
num_distinct,num_buckets,density,NUM_NULLS,AVG_COL_LEN,
to_char(last_analyzed,'YYYY-MM-DD HH24:MI:SS') "LAST_ANALYZED",
sample_size,data_default
from USER_TAB_COLS tc
where 
tc.table_name in ('SALES','PRODUCTS')
order by tc.table_name,column_id;

set autotrace on

-- query returns no rows
-- no cheese was sold in January 2012
-- but plan shows that query returns 2 rows

select 
sale_date, product_name, amount
from sales, products
where
sales.product_number=products.product_number and
sale_date between 
  to_date('01/01/2012','MM/DD/YYYY') and 
  to_date('01/31/2012','MM/DD/YYYY') and
product_type = 'Cheese';

spool off
